@@ -8,6 +8,7 @@ from serializers.renderers import (
     JSONRenderer,
     YAMLRenderer,
     XMLRenderer,
+    HTMLRenderer,
     CSVRenderer,
     DumpDataXMLRenderer
 )
@@ -92,6 +93,7 @@ class BaseSerializer(Field):
         'json': JSONRenderer,
         'yaml': YAMLRenderer,
         'csv': CSVRenderer,
+        'html': HTMLRenderer,
     }
 
     options_class = SerializerOptions
@@ -103,6 +105,8 @@ class BaseSerializer(Field):
         serialize = kwargs.get('serialize', None)
         super(BaseSerializer, self).__init__(source=source, label=label, serialize=serialize)
 
+        self.kwargs = kwargs
+        self.root = None
         self.opts = self.options_class(self.Meta, **kwargs)
         self.stack = []
         self.fields = SortedDict((key, copy.copy(field))
@@ -197,6 +201,7 @@ class BaseSerializer(Field):
         of state so that we can deal with handling maximum depth and recursion.
         """
         self.parent = parent
+        self.root = parent.root or parent
         self.orig_obj = obj
         self.orig_field_name = field_name
 
@@ -227,13 +232,20 @@ class BaseSerializer(Field):
             ret.set_with_metadata(key, value, field)
         return ret
 
+    def serialize_iterable(self, obj):
+        for item in obj:
+            yield self.serialize(item)
+
     def serialize(self, obj):
         if self._is_protected_type(obj):
             return obj
         elif self._is_simple_callable(obj):
             return self.serialize(obj())
+        elif isinstance(obj, dict):
+            return dict([(key, self.serialize(val))
+                         for (key, val) in obj.items()])
         elif hasattr(obj, '__iter__'):
-            return [self.serialize(item) for item in obj]
+            return self.serialize_iterable(obj)
         return self.serialize_object(obj)
 
     def encode(self, obj, format=None, **opts):
@@ -251,7 +263,7 @@ class Serializer(BaseSerializer):
     __metaclass__ = SerializerMetaclass
 
 
-class ModelSerializer(Serializer):
+class ModelSerializer(RelatedField, Serializer):
     """
     A serializer that deals with model instances and querysets.
     """
@@ -262,6 +274,11 @@ class ModelSerializer(Serializer):
         model_field_types = ('pk', 'fields', 'many_to_many')
 
     def get_default_field_names(self, obj):
+        """
+        We subclass this method to return the set of all field names defined
+        on the model instance, rather than the default behaviour of returning
+        all non-private attributes on the object.
+        """
         fields = []
         concrete_model = obj._meta.concrete_model
 
@@ -285,6 +302,10 @@ class ModelSerializer(Serializer):
         return self.opts.related_field()
 
     def get_flat_serializer(self, obj, field_name):
+        """
+        We subclass this method to switch between `related_field` and
+        `flat_field` depending on the field type.
+        """
         try:
             field = obj._meta.get_field_by_name(field_name)[0]
             if isinstance(field, RelatedObject) or field.rel:
@@ -293,17 +314,6 @@ class ModelSerializer(Serializer):
         except FieldDoesNotExist:
             return self.opts.flat_field()
 
-    def serialize(self, obj):
-        if self._is_protected_type(obj):
-            return obj
-        elif self._is_simple_callable(obj):
-            return self.serialize(obj())
-        elif hasattr(obj, 'all') and self._is_simple_callable(obj.all):
-            return [self.serialize(item) for item in obj.all()]
-        elif hasattr(obj, '__iter__'):
-            return [self.serialize(item) for item in obj]
-        return self.serialize_object(obj)
-
 
 class DumpDataFields(ModelSerializer):
     _use_sorted_dict = False
